<?php

/**
 * @file
 * Included file for the UserName Policy module.
 */

/**
 * Implements hook_form().
 */
function username_policy_admin_form($form, &$form_state) {
  $form['username_policy'] = array(
    '#type' => 'fieldset',
    '#title' => t('User Name Policy Configuration'),
    '#tree' => TRUE,
  );
  $username_policy = variable_get('username_policy', array());
  $form['username_policy']['pattern'] = array(
    '#type' => 'textfield',
    '#default_value' => isset($username_policy['pattern']) ? $username_policy['pattern'] : '',
    '#title' => t('Pattern for username'),
  );
  $form['username_policy']['note'] = array(
    '#markup' => t('Note:
                    <ul>
                    <li>Use only the below pattern elements to set pattern.</li>
                    <li>Only mandatory fields from registration page are allowed as pattern elements.</li>
                    <li>If pattern is empty username will be your email id.</li>
                    </ul>
                    <br/>
                 '),
  );
  $form['username_policy']['description'] = array(
    '#type' => 'item',
    '#title' => t('Only use the below pattern elements.'),
  );
  $form['username_policy']['token_tree'] = array(
    '#prefix' => '<div id="username_policy_token_tree">',
    '#suffix' => '</div>',
    '#tree' => TRUE,
    '#theme' => 'username_policy_pattern_elements',
  );
  foreach (username_policy_generate_pattern_fields() as $pattern_ele) {
    $form['username_policy']['token_tree'][] = array(
      '#markup' => $pattern_ele,
    );
  }
  return system_settings_form($form);
}

/**
 * Generate pattern fields.
 */
function username_policy_generate_pattern_fields() {
  $fieldinfo = field_info_instances('user', 'user');
  // Prepare pattern elements from user main profile.
  $_fields = array();
  foreach (array_keys($fieldinfo) as $entity) {
    if ($fieldinfo[$entity]['required'] && $fieldinfo[$entity]['settings']['user_register_form']) {
      $field_detail = field_info_field($entity);
      $field_type = $field_detail['type'];
      if ($field_type == 'text' || $field_type == 'number_integer') {
        $_fields[] = '[' . $entity . ']';
      }
    }
  }
  // Prepare pattern elements from user other profile.
  $fieldinfo_profile2 = NULL;
  $_fields_profile2 = array();
  if (module_exists('profile2')) {
    $fieldinfo_profile2 = field_info_instances('profile2');
    $result = db_query("SELECT type, data FROM {profile_type}");
    $data_registration_status = array();
    foreach ($result as $record) {
      $data_reg_status = unserialize($record->data);
      $data_registration_status[] = $data_reg_status['registration'];
    }
    foreach (array_keys($fieldinfo_profile2) as $entity) {
      foreach (array_keys($fieldinfo_profile2[$entity]) as $a) {
        $field_detail = field_info_field($a);
        $field_type = $field_detail['type'];
        if ($field_type == 'text' || $field_type == 'number_integer') {
          if ($fieldinfo_profile2[$entity][$a]['required']) {
            $_fields_profile2[] = '[' . $a . ']';
          }
        }
      }
    }
    for ($i = 0; $i < count($data_registration_status); $i++) {
      if ($data_registration_status[$i] == 0) {
        $_fields_profile2[$i] = '';
      }
    }
  }

  if (module_exists('profile2')) {
    foreach ($_fields_profile2 as $field) {
      $_fields[] = $field;
    }
  }
  return $_fields;
}

/**
 * Implements hook_form_validate().
 */
function username_policy_admin_form_validate($form_id, $form_state) {
  $pattern = $form_state['values']['username_policy']['pattern'];
  $offset = 0;
  $not_found = array();
  $num_of_patterns = substr_count($pattern, '[');
  for ($i = 0; $i <= $num_of_patterns - 1; $i++) {
    $str1 = strpos($pattern, '[', $offset);
    $str2 = strpos($pattern, ']', $str1);
    $length = ($str2 - $str1) - 1;
    $offset = $str2;
    $for_query = substr($pattern, ($str1 + 1), $length);
    $field_detail = field_info_field($for_query);
    $field_type = $field_detail['type'];
    $field_required = field_info_instance('user', $for_query, 'user');
    if ($field_required['required'] == 1) {
      if (isset($field_type) && ($field_type == 'text' || $field_type == 'number_integer')) {
        continue;
      }
    }
    $not_found[] = '[' . $for_query . ']';
  }
  if (count($not_found)) {
    form_set_error('username_policy_pattern_error', t('These not_found are non exist pattern sets!', array('not_found' => implode(', ', $not_found))));
  }
  if ($num_of_patterns == 0) {
    form_set_error('username_policy_pattern_error', t('Please enter any PATTERN ELEMENTS, which are shown below..!'));
  }
}

/**
 * Function to clear cache of user name pattern.
 */
function username_policy_cache_clear() {
  cache_clear_all('cache:variable_get:username_policy_pattern', 'cache');
}
